package more.ssaqms.xiamen.controller;

import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import more.commons.utils.response.ResponseResult;
import more.commons.utils.response.exception.MoreException;
import more.ssaqms.xiamen.handler.day.good.ReachGoodReport;
import more.ssaqms.xiamen.handler.month.airquality.AirQualityReport;
import more.ssaqms.xiamen.handler.month.supper.SupperMonthReport;
import more.ssaqms.xiamen.handler.month.traffic.TrafficMonthReport;
import more.ssaqms.xiamen.model.airquality.AirQualityInput;
import more.ssaqms.xiamen.model.reachGood.ReachGoodInput;
import more.ssaqms.xiamen.model.report.HtmlInput;
import more.ssaqms.xiamen.model.report.ReportOutput;
import more.ssaqms.xiamen.model.report.WordInput;
import more.ssaqms.xiamen.model.supper.SupperInput;
import more.ssaqms.xiamen.model.traffic.TrafficInput;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;

/**
 * @author zuo
 * @date 2024/8/28 15:56
 */

@Api(value = "报告接口")
@RestController
@RequestMapping(value = "/report")
@Slf4j
public class ReportController {

    @Autowired
    private ReachGoodReport reachGoodReport;

    @Autowired
    private AirQualityReport airQualityReport;

    @Autowired
    private SupperMonthReport supperMonthReport;

    @Autowired
    private TrafficMonthReport trafficMonthReport;

    @ApiOperation(value = "根据数据生成html", notes = "根据数据生成html", produces = "application/json")
    @PostMapping(value = "/html")
    public ResponseResult<String> getReachGoodReport(@RequestBody @Validated HtmlInput input) {
        String reportType = input.getReportType();
        ReportOutput output;
        switch (reportType){
            case "dldaily":
                ReachGoodInput reachGoodInput = JSON.parseObject(JSON.toJSONString(input.getData()), ReachGoodInput.class);
                output = reachGoodReport.creatDayHtmlReport(reachGoodInput);
                break;
            case "airquality":
                AirQualityInput airQualityInput = JSON.parseObject(JSON.toJSONString(input.getData()), AirQualityInput.class);
                output = airQualityReport.creatMonthHtmlReport(airQualityInput);
                break;
            case "super":
                SupperInput supperInput = JSON.parseObject(JSON.toJSONString(input.getData()), SupperInput.class);
                output = supperMonthReport.creatMonthHtmlReport(supperInput);
                break;
            case "traffic":
                TrafficInput trafficInput = JSON.parseObject(JSON.toJSONString(input.getData()), TrafficInput.class);
                output = trafficMonthReport.creatMonthHtmlReport(trafficInput);
                break;
            default:
                throw new MoreException("reportType参数不正确");
        }
        return ResponseResult.success(output.getHtml());
    }

    @ApiOperation(value = "根据html生成word", notes = "根据html生成word", produces = "application/octet-stream")
    @PostMapping(value = "/word")
    public void exportReport(@RequestBody @Validated WordInput input, HttpServletResponse response) {
        String reportType = input.getReportType();
        String html = input.getHtml();
        switch (reportType){
            case "dldaily":
                reachGoodReport.exportMonthReport(html,response);
                break;
            case "airquality":
                airQualityReport.exportMonthReport(html,response);
                break;
            case "super":
                supperMonthReport.exportMonthReport(html,response);
                break;
            case "traffic":
                trafficMonthReport.exportMonthReport(html,response);
                break;
            default:
                throw new MoreException("reportType参数不正确");
        }
    }
}
