package more.ssaqms.xiamen.handler.day.good;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.extern.slf4j.Slf4j;
import more.commons.utils.response.exception.MoreException;
import more.ssaqms.xiamen.handler.ReportDataUtil;
import more.ssaqms.xiamen.model.reachGood.ReachGoodInput;
import more.ssaqms.xiamen.model.report.ReportOutput;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.WordTransformUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;

@Component
@Slf4j
public class ReachGoodReport {

    @Value("${cloud.report.path}")
    private String reportPath;

    @Value("${cloud.report.chart.url}")
    private String chartUrl;

    @Autowired
    private ReachGoodChapter chapter;

    @Autowired
    private ReachGoodChapter1 chapter1;

    @Autowired
    private ReachGoodChapter2 chapter2;

    @Autowired
    private ReachGoodChapter3 chapter3;

    @Autowired
    private ReachGoodChapter4 chapter4;

    @Autowired
    private ReachGoodChapter5 chapter5;

    @Autowired
    private ReachGoodChapter6 chapter6;

    @Transactional
    public ReportOutput creatDayHtmlReport(ReachGoodInput input) {
        String path = "/image/";
        //序号，0位图，1为表
        Map<String, Integer> countMap = new HashMap<>();
        countMap.put("imageCount", 1);
        countMap.put("tableCount", 1);
        TimeRange dayTimeRange = ReportDataUtil.getDayTimeRange(input.getDate());

        String chapter = this.chapter.html(dayTimeRange);

        String chapter1 = this.chapter1.html(dayTimeRange, input.getAqiData(), countMap);

        String chapter2 = this.chapter2.html(dayTimeRange, input.getWeatherData(), countMap);

        String chapter3 = this.chapter3.html(dayTimeRange, countMap, input.getVocData());

        String chapter4 = this.chapter4.html(dayTimeRange, countMap, input.getParticleData());

        String chapter5 = this.chapter5.html(dayTimeRange, countMap, input.getLidarData());

        String chapter6 = this.chapter6.html(dayTimeRange, input.getAqiData(), input.getWeatherData(), input.getVocData(), input.getParticleData());
//        String chapter6 = "";
        ReportOutput reportOutput = new ReportOutput();
        reportOutput.setHtml(chapter + chapter1 + chapter2 + chapter3 + chapter4 + chapter5 + chapter6);
        reportOutput.setDate(input.getDate());
        return reportOutput;
    }

    public void exportMonthReport(@RequestBody @Validated String input, HttpServletResponse response) {
        String localHtmlStream = WordTransformUtils.dayReportHtmlAll(input);
        String idStr = IdWorker.getIdStr();
        String path = reportPath + "ReachGoodReport/" + idStr;
        String filePathStr = WordTransformUtils.writeWordFile(localHtmlStream, path, path + "/", idStr, false, null);

        File file = new File(filePathStr);
        if (!file.exists()) {//如果文件不存在
            throw new MoreException("文件未找到");
        }
        String fileName = "dldaily.docx";
        //输入处理流
        InputStream inputStream = null;
        //输出处理流
        OutputStream outputStream = null;
        try {
            //定义处理流
            inputStream = new BufferedInputStream(Files.newInputStream(file.toPath()));
            outputStream = new BufferedOutputStream(response.getOutputStream());

            response.reset();
            //解决跨域文件名问题
            response.addHeader("Access-Control-Expose-Headers", "content-disposition");
            // 设置在下载框默认显示的文件名
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            //写明文件大小
            response.addHeader("Content-Length", "" + file.length());
            // 指明response的返回对象是文件流
            response.setContentType("application/octet-stream");
            //response.setContentType("multipart/form-data");

            //"开始读取文件流"
            //返回从该输入流中可以读取（或跳过）的字节数的估计值，而不会被下一次调用此输入流的方法阻塞
            byte[] bytes = new byte[inputStream.available()];
            //将pdf的内容写入bytes中
            inputStream.read(bytes);
            //将bytes中的内容写入
            outputStream.write(bytes);
            //刷新输出流，否则不会写出数据
            outputStream.flush();

            //文件下载处理完成
            FileUtils.delFile(new File(path));


        } catch (FileNotFoundException e) {
            log.error("文件未找到" + e.getMessage());
        } catch (IOException e) {
            log.error("文件读写异常" + e.getMessage());
        } finally {//关闭输入流、输出流
            try {
                if (inputStream != null) {
                    inputStream.close();
                    log.debug("输入流关闭成功！");
                }
                if (outputStream != null) {
                    outputStream.close();
                    log.debug("输出流关闭成功！");
                }
            } catch (IOException e) {
                log.error("关闭处理流出现异常" + e.getMessage());
            }
        }
    }
}
