package more.ssaqms.xiamen.handler.month.airquality;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.extern.slf4j.Slf4j;
import more.commons.utils.response.exception.MoreException;
import more.ssaqms.xiamen.handler.ReportDataUtil;
import more.ssaqms.xiamen.model.airquality.AirQualityInput;
import more.ssaqms.xiamen.model.airquality.SiteAqiData;
import more.ssaqms.xiamen.model.airquality.SiteComplexData;
import more.ssaqms.xiamen.model.report.ReportOutput;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.supper.htmlInput;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.WordTransformUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.*;

@Component
@Slf4j
public class AirQualityReport {

    @Value("${cloud.report.path}")
    private String reportPath;
    @Autowired
    private AirQualityChapter airQualityChapter;

    @Autowired
    private AirQualityChapter1 airQualityChapter1;

    @Autowired
    private AirQualityChapter2 airQualityChapter2;

    @Autowired
    private AirQualityChapter3 airQualityChapter3;

    @Transactional
    public ReportOutput creatMonthHtmlReport(AirQualityInput input) {
        TimeRange monthTimeRange = ReportDataUtil.getMonthTimeRange(input.getDate());

        //常规污染物监测
        List<SiteAqiData> siteAqiDatas = input.getSiteAqiDatas();

        List<SiteAqiData> monthSiteAqiDatas = input.getMonthSiteAqiDatas();

        List<SiteAqiData> regionSiteAqiDatas = input.getRegionSiteAqiDatas();

        List<SiteAqiData> monthRegionSiteAqiDatas = input.getMonthRegionSiteAqiDatas();

        String html = airQualityChapter.html(monthTimeRange);
        //第一段
        String html1 = airQualityChapter1.html(monthTimeRange, siteAqiDatas, monthSiteAqiDatas);
        //第二段
        String html2 = airQualityChapter2.html(monthTimeRange, regionSiteAqiDatas, monthRegionSiteAqiDatas);
        //附表1、2
        String html3 = airQualityChapter3.html(monthTimeRange, siteAqiDatas, monthSiteAqiDatas, 1, 2, "厦门市各国控点空气质量情况");
        //附表3、4
        String html4 = airQualityChapter3.htmlSort(monthTimeRange, regionSiteAqiDatas, monthRegionSiteAqiDatas, 3, 4, "厦门市各行政区环境空气质量情况");
        ReportOutput reportOutput = new ReportOutput();
        reportOutput.setDate(input.getDate());
        reportOutput.setHtml(html + html1 + html2 + html3 + html4);
        return reportOutput;
    }

    public void exportMonthReport(@RequestBody @Validated String input, HttpServletResponse response) {
        String localHtmlStream = WordTransformUtils.dayReportHtmlAll(input);
        String idStr = IdWorker.getIdStr();
        String path = reportPath + "AirQualityReport/" + idStr;
        String filePathStr = WordTransformUtils.writeWordFile(localHtmlStream, path, path + "/", idStr, false, null);

        File file = new File(filePathStr);
        if (!file.exists()) {//如果文件不存在
            throw new MoreException("文件未找到");
        }
        String fileName = "airquality.docx";
        //输入处理流
        InputStream inputStream = null;
        //输出处理流
        OutputStream outputStream = null;
        try {
            //定义处理流
            inputStream = new BufferedInputStream(Files.newInputStream(file.toPath()));
            outputStream = new BufferedOutputStream(response.getOutputStream());

            response.reset();
            //解决跨域文件名问题
            response.addHeader("Access-Control-Expose-Headers", "content-disposition");
            // 设置在下载框默认显示的文件名
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            //写明文件大小
            response.addHeader("Content-Length", "" + file.length());
            // 指明response的返回对象是文件流
            response.setContentType("application/octet-stream");
            //response.setContentType("multipart/form-data");

            //"开始读取文件流"
            //返回从该输入流中可以读取（或跳过）的字节数的估计值，而不会被下一次调用此输入流的方法阻塞
            byte[] bytes = new byte[inputStream.available()];
            //将pdf的内容写入bytes中
            inputStream.read(bytes);
            //将bytes中的内容写入
            outputStream.write(bytes);
            //刷新输出流，否则不会写出数据
            outputStream.flush();

            //文件下载处理完成
            FileUtils.delFile(new File(path));


        } catch (FileNotFoundException e) {
            log.error("文件未找到"+e.getMessage());
        } catch (IOException e) {
            log.error("文件读写异常"+e.getMessage());
        }finally {//关闭输入流、输出流
            try {
                if (inputStream != null){
                    inputStream.close();
                    log.debug("输入流关闭成功！");
                }
                if (outputStream != null){
                    outputStream.close();
                    log.debug("输出流关闭成功！");
                }
            } catch (IOException e) {
                log.error("关闭处理流出现异常"+e.getMessage());
            }
        }
    }
}
