package more.ssaqms.xiamen.handler.month.supper;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.extern.slf4j.Slf4j;
import more.commons.utils.response.exception.MoreException;
import more.ssaqms.xiamen.handler.ReportDataUtil;
import more.ssaqms.xiamen.model.report.ReportOutput;
import more.ssaqms.xiamen.model.report.TimeRange;
import more.ssaqms.xiamen.model.supper.*;
import more.ssaqms.xiamen.util.FileUtils;
import more.ssaqms.xiamen.util.WordTransformUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author zuo
 * @date 2023/5/4 12:21
 */
@Component
@Slf4j
public class SupperMonthReport {

    @Value("${cloud.report.path}")
    private String reportPath;

    @Autowired
    private SupperHomePage supperHomePage;

    @Autowired
    private SupperChapter1 chapter1;

    @Autowired
    private SupperChapter2 chapter2;

    @Autowired
    private SupperChapter3 chapter3;

    @Autowired
    private SupperChapter4 chapter4;


    @Transactional
    public ReportOutput creatMonthHtmlReport(SupperInput input) {
        Date date = input.getDate();
        if (Objects.isNull(date)) {
            ReportOutput output = new ReportOutput();
            output.setHtml("");
            return output;
        }
        TimeRange monthTimeRange = ReportDataUtil.getMonthTimeRange(date);
        //常规污染物监测
        List<SiteAqiData> siteAqiDataList = input.getSiteAqiDataList();
        String home = supperHomePage.html(monthTimeRange);

        String chapter1 = this.chapter1.html(monthTimeRange, siteAqiDataList);

        List<OcEcData> ocEcDataList = input.getOcEcDataList();
        List<HeavyMetalData> heavyMetalDataList = input.getHeavyMetalDataList();
        List<BlackCarbonData> blackCarbonDataList = input.getBlackCarbonDataList();
        List<IonData> ionDataList = input.getIonDataList();
        List<SpectrometryData> spectrometryDataList = input.getSpectrometryDataList();
        List<LidarData> lidarDataList = input.getLidarDataList();

        String chapter2 = this.chapter2.html(ocEcDataList, heavyMetalDataList, blackCarbonDataList, ionDataList, lidarDataList, spectrometryDataList);

        List<HtcData> htcDataList = input.getHtcDataList();
        List<VocsData> vocsDataList = input.getVocsDataList();
        List<OzoneRadarData> ozoneRadarDataList = input.getOzoneRadarDataList();
        List<WeatherData> weatherDataList = input.getWeatherDataList();

        String chapter3 = this.chapter3.html(htcDataList, vocsDataList, ozoneRadarDataList, weatherDataList);

        String chapter4 = this.chapter4.html(siteAqiDataList, ocEcDataList, heavyMetalDataList, blackCarbonDataList, spectrometryDataList, htcDataList, vocsDataList, weatherDataList);
        ReportOutput reportOutput = new ReportOutput();
        reportOutput.setDate(input.getDate());
        reportOutput.setHtml(home + chapter1 + chapter2 + chapter3 + chapter4);
        return reportOutput;
    }

    public void exportMonthReport(String input, HttpServletResponse response) {
        String localHtmlStream = WordTransformUtils.dayReportHtmlAll(input);
        String idStr = IdWorker.getIdStr();
        String path = reportPath + "SupperReport/" + idStr;
        String filePathStr = WordTransformUtils.writeWordFile(localHtmlStream, path, path + "/", idStr, false, null);


        File file = new File(filePathStr);

        if (!file.exists()) {//如果文件不存在
            throw new MoreException("文件未找到");
        }
        String fileName = "supper.docx";
        //输入处理流
        InputStream inputStream = null;
        //输出处理流
        OutputStream outputStream = null;
        try {
            //定义处理流
            inputStream = new BufferedInputStream(Files.newInputStream(file.toPath()));
            outputStream = new BufferedOutputStream(response.getOutputStream());

            response.reset();
            //解决跨域文件名问题
            response.addHeader("Access-Control-Expose-Headers", "content-disposition");
            // 设置在下载框默认显示的文件名
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            //写明文件大小
            response.addHeader("Content-Length", "" + file.length());
            // 指明response的返回对象是文件流
            response.setContentType("application/octet-stream");
            //response.setContentType("multipart/form-data");

            //"开始读取文件流"
            //返回从该输入流中可以读取（或跳过）的字节数的估计值，而不会被下一次调用此输入流的方法阻塞
            byte[] bytes = new byte[inputStream.available()];
            //将pdf的内容写入bytes中
            inputStream.read(bytes);
            //将bytes中的内容写入
            outputStream.write(bytes);
            //刷新输出流，否则不会写出数据
            outputStream.flush();

            //文件下载处理完成
            FileUtils.delFile(new File(path));


        } catch (FileNotFoundException e) {
            log.error("文件未找到" + e.getMessage());
        } catch (IOException e) {
            log.error("文件读写异常" + e.getMessage());
        } finally {//关闭输入流、输出流
            try {
                if (inputStream != null) {
                    inputStream.close();
                    log.debug("输入流关闭成功！");
                }
                if (outputStream != null) {
                    outputStream.close();
                    log.debug("输出流关闭成功！");
                }
            } catch (IOException e) {
                log.error("关闭处理流出现异常" + e.getMessage());
            }
        }
    }


}
